%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the M-files of a simulation in the paper entitled
% Global asymmptotic stabilizing feedback law for a system
% satifying two different sector conditions
% written by V. Andrieu, C Prieur, S. Tarbouriech, D. Arzelier
%
% Associated to Example 1
%
% Last modification 13/07/2010 by V. Andrieu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Definition of the model and initialization of the data
clear all;
close all;

% setting of the output
% To display the value of the uniting controler found
Disp_Unit = 0;
% To display the figures of the local, non-local and global controller
Disp_Graph = 1; 

% settings of the solver
opt = sdpsettings('solver','sedumi','verbose',0);

% The system is defined as : 
% dot x = Ax + Bu + G phi
% where 
% phi is a function defined in the M-file Phi.m
global A B G Ti Si S0 v0 vi;
A = [0 5; 0 0];
B = [0 1]';
G = [1 0; 0 1];

Ti =[-1; -6];
Si = [4 0; 0 4];
S0 = [0.1 0; 0 0.1];

% The function Phi satisfies the sector bound conditions :
% phi'(phi-S0 x)<0 for |x| < v0
% phi'(phi-Si x - Ti)<0 for |x| > vi
v0 = 14;
vi = 2;
% We may check that the function satisfies the Sector bound condition
TestSectCond;


% We store the size of the matrices
[n,m] = size(B);
[n,p] = size(G);

%% Design of the local Controller
% Following Proposition 2.1 in the paper, we can formulize the asymptotic
% stabilization problem in terms of LMI.

% Definition of the unknown variables of the LMI test
W0 = sdpvar(n,n,'symmetric');
H0 = sdpvar(m,n,'full');
J0 = sdpvar(m,p,'full');
eta0 = sdpvar(1,1,'full');

% Matrix that has to be negative for the local design
Mat0 = [
    [A*W0 + B*H0  + (A*W0)'+ (B*H0)'  B*J0 + G*eta0 + W0 * S0']
    [(B*J0 + G*eta0 + W0 * S0')' -2*eye(p)*eta0]
    ];


%% Design of a nonlocal controller
% Following Proposition 2.2 in the paper, we can formulize the non local
% asymptotic stabilization problem in terms of LMI.

% Definition of the unknown variables of the LMI test
Wi = sdpvar(n,n,'symmetric');
Hi = sdpvar(m,n,'full');
Ji = sdpvar(m,p,'full');
etai = sdpvar(1,1,'full');
taui = sdpvar(1,1,'full');
Zm = sdpvar(1,1,'full');
Li = sdpvar(m,m,'symmetric');
% Note that in the paper Li is a diagonal matrix, since we don't know how to
% implement in Yalmip a diagonal unknown, instead we use the following
% trick
% ei = [0 ... 1 0;
% In the LMI, Li will be employed in the form 
% dLi = sum ei'ei Li * ei'ei;
% which will preserve only the diagonal terms
% Note however that in our case, Li is real and already diagonal.
dLi = Li;

% The one matrix is also defined
One = 1;

% Matrix that has to be negative for the Non-local design
Mati = [
    [A*Wi+B*Hi+(A*Wi)'+(B*Hi)'  B*Ji+G*etai+Wi*Si' B*dLi+G*Ti*Zm Wi ]
    [(B*Ji+G*etai+Wi*Si')' -2*etai*eye(p) Ti*Zm zeros(p,n)]
    [(B*dLi+G*Ti*Zm)' (Ti*Zm)' -taui*vi^2/2*eye(m) zeros(m,n)]
    [Wi zeros(n,p) zeros(n,m) -taui*eye(2)]
    ];
% Equality constraint
Equal1 = Zm*One-taui;

%% LMI TEST FOR THE TWO STEPS DESIGN

% We test the LMI for the local controller
LMI =  [Mat0<0, W0>0];
solvesdp(LMI, [], opt);

% we check if a solution for the Local Controller was solved
if max(eig(double(Mat0)))>0 
    % In that case the LMI couldn't be solved
    disp('Problem with the local controller');
    return
end
disp('Local controller found.');

% We test the LMI for the Non-local controller
LMI =  [Mati<0, Wi>0, Equal1==0];
solvesdp(LMI, [], opt);

% we check if a solution for the Non-local Controller was obtained
if max(eig(double(Mati)))>0 
    % In that case the LMI couldn't be solved
    disp('Problem with the non-local controller');
    return
end
disp('Non-local controller found.');


%%  Test of the covering Assumption
P0 = inv(double(W0));
R0 = min(eig(P0))*v0^2;
Pi = inv(double(Wi));
ri = max(eig(Pi))*vi^2;
if max(eig(ri*P0-R0*Pi))>0
    disp('The covering Assumption failed to work with these data')
    % Design including the Test of the covering assumption following
    % Proposition 4.1
    rho = sdpvar(1,1,'full');
    LMICA_0 = W0 - rho * v0*v0 * eye(n);
    LMICA_i = rho * vi*vi * eye(n) - Wi;
    LMICA = Wi-W0;
    LMI =  [Mat0<0, W0>0, Mati<0, Wi>0, Equal1==0, LMICA<0, LMICA_0<0, LMICA_i<0];
    solvesdp(LMI, [], opt);
    % we check if the LMI was solved
    if max([max(eig(double(Mat0))), max(eig(double(Mati))), max(eig(double(LMICA))), max(eig(double(LMICA_0))), max(eig(double(LMICA_i)))])>0 
        % In that case the LMI couldn't be solved
        disp('Problem with the covering assumption in  one step');
        %%%%%% we stop.
        return
    else
        % In that case the LMI was solved
        disp('The covering Assumption is satisfied with the modified design')
    end    
else
    % In that case the LMI was solved
    disp('The covering Assumption is satisfied')

end
% we can draw the level ring
P0 = inv(double(W0));
R0 = min(eig(P0))*v0^2;
Pi = inv(double(Wi));
ri = max(eig(Pi))*vi^2;
% New figure showing the Covering assumption
scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(3)/6 scrsz(3)/6 scrsz(3)/3 scrsz(4)/3],'name','Covering Assumption')
LevelCurve2d(P0,R0,100);
hold on;
LevelCurve2d(Pi,ri,100);
hold on;
LevelCurve2d(eye(2),v0^2,100);
hold on;
LevelCurve2d(eye(2),vi^2,100);


%% Display output
K0 = double(H0)/(double(W0));
J0 = double(J0);
eta0 = double(eta0);
disp('The local controller is u=K0x + J0 phi/eta0 with');
disp(['K0 = ',num2str(K0)]);
disp(['J0 = ',num2str(J0)]);
disp(['eta0 = ',num2str(eta0)]);
disp('The Lyapunov function is xTP0x with');
P0 = inv(double(W0))
disp('With bassin of attraction xTPOx<=R0 with')
R0 = min(eig(P0))*v0^2;
disp(['R0 = ',num2str(R0)]);
Ki = double(Hi)/(double(Wi));
Ji = double(Ji);
etai = double(etai);
taui = double(taui);
Li = double(dLi);
disp('The non-local controller is u=Ki x + Ji phi/etai +  Li/taui, where');
disp(['Ki = ',num2str(Ki)]);
disp(['Ji = ',num2str(Ji)]);
disp(['etai = ',num2str(etai)]);
disp(['taui = ',num2str(taui)]);
disp(['Li = ',num2str(Li)]);
disp('The Lyapunov function is xTPix with');
Pi = inv(double(Wi))
ri = max(eig(Pi))*vi^2;
disp('With bassin of attraction xTPix>=ri with')
disp(['ri = ',num2str(ri)]);


disp('Test for the asymptotic behavior with the local controller');
eig(A+B*K0+(B*J0/eta0+G)*Si/2)



%% Test for the uniting controller design

Jm = sdpvar(m,p,'full');
Km = sdpvar(m,n,'full');
Lm = sdpvar(1,1);
mu1 = sdpvar(1,1);
mu2 = sdpvar(1,1);
theta1 = sdpvar(1,1);
theta2 = sdpvar(1,1);
nu1 = sdpvar(1,1);
nu2 = sdpvar(1,1);
nu3 = sdpvar(1,1);
nu4 = sdpvar(1,1);
Z11 = sdpvar(1,m);
Z12 = sdpvar(1,m);
Z21 = sdpvar(1,m);
Z22 = sdpvar(1,m);

Q0 = [
    [zeros(n,n) (-S0') zeros(n,m)]
    [(-S0) 2*eye(p) zeros(p,m)]
    [zeros(m,n) zeros(m,p) zeros(m,m)]
    ];
Qi1 = [
    [zeros(n,n) (-theta1*S0') zeros(n,m)]
    [(-theta1*S0) 2*theta1*eye(p) -Ti*Z12]
    [zeros(m,n) -Ti'*Z12 zeros(m,m)]
    ];
Qi2 = [
    [zeros(n,n) (-theta2*S0') zeros(n,m)]
    [(-theta1*S0) 2*theta2*eye(p) -Ti*Z22]
    [zeros(m,n) -Ti'*Z22 zeros(m,m)]
    ];

MatUnit0 = [
    [P0*(A+B*Km)+(A+B*Km)'*P0-nu1*P0+nu2*Pi P0*(B*Jm+G) Lm*P0*B]
    [(B*Jm+G)'*P0 zeros(p,p) zeros(p,m)]
    [Lm* B'*P0' zeros(m,p) (nu1*R0-nu2*ri)/m*eye(m)]
    ] - mu1*Q0- Qi1;
MatUniti = [
    [Pi*(A+B*Km)+(A+B*Km)'*Pi+nu4*Pi-nu3*P0 Pi*(G+B*Jm) Lm*Pi*B]
    [(B*Jm+G)'*Pi zeros(p,p) zeros(p,m)]
    [Lm*B'*Pi zeros(m,p) (nu3*R0-nu4*ri)/m*eye(m)]
    ] - mu2*Q0-Qi2;

% Equality constraints
Eq12 = Z12*One-theta1;
Eq22 = Z22*One-theta2;


LMIUnit =[MatUnit0 < 0, MatUniti < 0, Eq12==0, Eq22==0];
solvesdp(LMIUnit, [], opt);
if max(eig(double(MatUnit0)), eig(double(MatUniti)))>0 
    disp('Problem with the Uniting LMI condition');
    return
end

% In that case the LMI was solved
disp('Uniting LMI in two steps satisfied')
Km = double(Km);
Jm = double(Jm);
Lm = double(Lm);
mu1 = double(mu1);
mu2 = double(mu2);
theta1 = double(theta1);
theta2 = double(theta2);
nu1 = double(nu1);
nu2 = double(nu2);
nu3 = double(nu3);
nu4 = double(nu4);
Z11 = double(Z11);
Z12 = double(Z12);
Z21 = double(Z21);
Z22 = double(Z22);

if Disp_Unit ==1
    disp('The solution is');
    disp(['Km = ',num2str(Km)]);
    disp(['Jm = ',num2str(Jm)]);
    disp(['Lm = ',num2str(Lm)]);
    disp(['mu1 = ',num2str(mu1)]);
    disp(['mu2 = ',num2str(mu2)]);
    disp(['theta1 = ',num2str(theta1)]);
    disp(['theta2 = ',num2str(theta2)]);
    disp(['nu1 = ',num2str(nu1)]);
    disp(['nu2 = ',num2str(nu2)]);
    disp(['nu3 = ',num2str(nu3)]);
    disp(['nu4 = ',num2str(nu4)]);
    disp(['Z11 = ',num2str(Z11)]);
    disp(['Z12 = ',num2str(Z12)]);
    disp(['Z21 = ',num2str(Z21)]);
    disp(['Z22 = ',num2str(Z22)]);
end


if Disp_Graph ==1
    % Figure of the Loc Controller
    LocContExample1;
    % Figure of the Non-Loc Controller
    NonLocContExample1;
    % Figure of the global Contoller
    GlobControlerExample1;
end
