%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    This is the M-files of a simulation in the paper entitled
% Global asymmptotic stabilizing feedback law for a system
% satifying two different sector conditions
%
% Associated to Example 1
%
% Last Modification 13/07/2010 by V. Andrieu
%
% In this M-file we integrate the model employing the United Global
% controller developped following the procedure of the paper
%
%% Parameter of the integration

% Data of the integration
TSimu = 10; % Time of the simulation
dt = 0.001; % Stepsize of the integration
NbIter = TSimu/dt;
% Vector of time    
t = zeros(NbIter,1);

% Parameter of the size of the arrow in term of iterations
SizeArrow = 1;
SizeEsp = 2.5*SizeArrow;

%Intial value of the state
x10 = -25;
x20 = -35;

% Parameter describing the size of the zoom
Xm=30;
Ym=Xm;

NbInitVal = size(x10);

%% Parameter of the Control Law
% Selection of the parameters which separate the state space for the
% uniting CLF problem.
% ri x'P0x \leq r0 x'Pix, 
r0 = 150;% ri * max(eig(P0)) / min(eig(Pi));
% Ri x'P0x \leq R0 x'Pix, 
Ri = 190;%R0 * min(eig(Pi)) / max(eig(P0));

% Selection of the high-gain parameter of the controller
k = 1;

% Initialization of the state vector for the united contoller
xv = zeros(2,NbIter);
xv(:,1) = [x10; x20];

% New figure showing the global controller
scrsz = get(0,'ScreenSize');
figure('Position',[1 scrsz(4)/2 scrsz(3)/3 scrsz(4)/3],'name','United Global Controller')

%% Integration of the model
for ii=1: NbIter-1

    %%%%%%%%%%%%%%%% Integration when using the united contoller
    x = xv(:,ii);
    x1 = x(1);
    x2 = x(2);
    % Computation of LgV     
    % First we compute
    % LgV0 = 2x'P0B
    % LgVi = 2x'PiB
    LgV0 = 2*x'*P0*B;
    LgVi = 2*x'*Pi*B;


    % The Non local controler is :
    ui=Ki*x + Ji*Phi(x1,x2)/etai + Li/taui;
    % The local one is 
    u0=K0*x + J0*Phi(x1,x2)/eta0;
    
    % The Lyapunov functions are
    V0 = x'*P0*x;
    Vi = x'*Pi*x;
    % we have
    % LgV = A LgV0 + B LgVi
    % with 
    % A = [R0 Vi - ri x'P0x] vphi0'(V0) + ri[1 - vphi0(V0) - vphii(Vi)] 
    % B = [R0 Vi - ri x'P0x] vphii'(Vi) + R0[vphi0(V0) + vphii(Vi)]
    % with 
    if V0 < r0
        vphi0=0;
        vphi0p=0;
    else if V0 > R0
            vphi0 = 1/2;
            vphi0p = 0;
        else
            vphi0 = 3/2 * ((V0-r0)/(R0-r0))^2 - ((V0-r0)/(R0-r0))^3;
            vphi0p = 3 / (R0-r0) * ((V0-r0)/(R0-r0) - ((V0-r0)/(R0-r0))^2);
        end
    end
    
    if Vi < ri
        vphii=0;
        vphiip=0;
    else if Vi > Ri
            vphii = 1/2;
            vphiip = 0;
        else
            vphii = 3/2 * ((Vi-ri)/(Ri-ri))^2 - ((Vi-ri)/(Ri-ri))^3;
            vphiip = 3 / (Ri-ri) * ((Vi-ri)/(Ri-ri) - ((Vi-ri)/(Ri-ri))^2);   
        end
    end
    %
    Ar = [R0 *Vi - ri *V0] *vphi0p + ri*[1 - vphi0 - vphii];
    Br = [R0 *Vi - ri *V0] *vphiip + R0*[vphi0 + vphii];
    LgV = Ar * LgV0 + LgVi * Br;
    
    %
    % The control law is given as :
    % gamma(x)phi0(x) + (1-gamma(x))phii - k * c(x)* LgV
    % with gamma defined as :
    gamma = 1-vphii-vphi0;
    cx = max(0, min((R0-V0)*(Vi-ri),1))/(1+abs(LgV));
    
    u = gamma*u0 + (1-gamma)*ui-k*cx*LgV;
        
    % Integration of the system with the control
    % Integration of the model using a simple Euler
    dxdt = A*x + B*u + G*Phi(x1,x2);
    xv(:,ii+1) = x + dxdt*dt;    
    
end

% Vector of time    
t = zeros(NbIter,1);
plot(xv(1,:),xv(2,:),'r','linewidth',1);

% Display of the output
d=0;
for jj=2:NbIter
        norme=sqrt((xv(1,jj)-xv(1,jj-1))^2 + (xv(2,jj)-xv(2,jj-1))^2);
        d=d+norme;
        if d>SizeEsp
            d=0;        
            
            x1 = xv(1,jj);
            y1 = x1 + (xv(1,jj) - xv(1,jj-1))/norme * SizeArrow;
            x2 = xv(2,jj);
            y2 = x2 +(xv(2,jj) - xv(2,jj-1))/norme * SizeArrow;
        
            f1 = [y1, x1 + 2/3*(y1-x1) - 1/3*(y2-x2)];
            f2 = [y2, x2 + 2/3*(y2-x2) + 1/3*(y1-x1)];
        
            g1 = [y1, x1 + 2/3*(y1-x1) + 1/3*(y2-x2)];
            g2 = [y2, x2 + 2/3*(y2-x2) - 1/3*(y1-x1)];
            hold on;
            plot([x1,y1],[x2,y2],'linewidth',1);
            hold on;    
            plot(f1,f2,'linewidth',1);    
            hold on;
            plot(g1,g2,'linewidth',1);
            AXIS([-Xm +Xm -Ym +Ym])
        end
end
    
hold on;
LevelCurve2d(Pi,ri,100);
hold on;
plot([Xm,-Xm],[0,0], '--k');
hold on;
plot([0,0],[Ym,-Ym], '--k');
hold on;
LevelCurve2d(P0,R0,100); 
    
