%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the M-files of a simulation in the paper entitled
% Global asymmptotic stabilizing feedback law for a system
% satifying two different sector conditions
% written by V. Andrieu, C Prieur, S. Tarbouriech, D. Arzelier
%
% Associated to Example 2
%
% Last modification 13/07/2010 by V. Andrieu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
% Matrices defining the system
A = zeros(3,3);
B = [1 0 0]';
G = eye(3);
L = eye(3);

% Size of the matrices
[n,m] = size(B);
q = size(L,2);
p = size(G,1);

% Sector conditions defining the nonlinearities
lambda0 = 0.6;
lambdai = 0.4;
Mi = [[0 0 0; 0 0 1; 1 0 0]];
N0 = [[0 0 0; 1 0 0; 0 1 0]];
M0 = lambda0 * Mi + (1-lambda0) * N0;
Ni = lambdai * Mi + (1-lambdai) * N0;

% The two parameters defining the sets in which the sector conditions hold
v0 = 10; % i.e. the local sector condition |x|<v0

vi = 1.5; % i.e. the non-local sector condition |x|>vi

% Exemple interressant : 
% v0 = 10, vi = 1.5 
% dans ce cas, l'approche R0 = ri = 1 ne marche pas alors que 1/rho = R0 = ri
% avec rho nouvelle vaviable a chercher fonctionne

% settings of the solver
opt = sdpsettings('solver','sedumi','verbose',0);

% option_computation == 1, we first modify v0 and vi 
% option_computation == 2, we first modify R0 and ri 
option_computation = 2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Test Usual Approach %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Local change of matvices (to get a LMI)
A_UA = A + G * Mi * L;
S_UA = N0 - Mi;
W_UA = sdpvar(n,n,'symmetric');
H_UA = sdpvar(m,n,'full');
J_UA = sdpvar(m,p,'full');

% Matvix that has to be negative for the local design
Mat_UA = [[A_UA*W_UA + B*H_UA  + (A_UA*W_UA)'+ (B*H_UA)'  B*J_UA + G + W_UA * L' * S_UA']
    [(B*J_UA + G + W_UA * L' * S_UA')' -2*eye(p)]];

LMI_UA =  [Mat_UA<0, W_UA>0]; 
solvesdp(LMI_UA);

% we check if the local LMI was solved
if max(eig(double(Mat_UA)))>0 disp('The usual approach does not work') 
else
    disp('ALERT : Usual approach can be applied')
end
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Local controller %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Local change of matvices (to get a LMI)
A0 = A + G * M0 * L;
S0 = N0 - M0;
W0 = sdpvar(n,n,'symmetric');
H0 = sdpvar(m,n,'full');
J0 = sdpvar(m,p,'full');

% Matrix that has to be negative for the local design
Mat0 = [[A0*W0 + B*H0  + (A0*W0)'+ (B*H0)'  B*J0 + G + W0 * L' * S0']
    [(B*J0 + G + W0 * L' * S0')' -2*eye(p)]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Set-Global Controller %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% change of matrices (to get a LMI)
Ai = A + G * Mi * L;
Si = Ni - Mi;
Wi = sdpvar(n,n,'symmetric');
Hi = sdpvar(m,n,'full');
Ji = sdpvar(m,p,'full');

% Matvix that has to be negative for the Set-Global design
Mati = [[Ai*Wi + B*Hi  + (Ai*Wi)'+ (B*Hi)'  B*Ji + G + Wi * L' * Si']
    [(B*Ji + G + Wi * L' * Si')' -2*eye(p)]];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Coveving Assumption %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
R0 = 1; % Dans ce cas on se donne une valeur particuliere pour la courbe de niveau -> sans doute restrictif
ri = 1; 
LMICA = Wi - W0;
LMICA_0 = W0-R0*v0*v0*eye(n);
LMICA_i = ri*vi*vi*eye(n)-Wi;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute LMI %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

LMI =  [Mat0<0, W0>0, Mati<0, Wi>0, LMICA < 0, LMICA_0 < 0, LMICA_i<0];
solvesdp(LMI, [], opt);

% we check if the LMI was solved
if max([max(eig(double(Mat0))), max(eig(double(Mati))), max(eig(double(LMICA))), max(eig(double(LMICA_0))), max(eig(double(LMICA_i)))])>0 
    % In that case the LMI couldn't be solved
    disp('Problem with the LMI');
    %%%%%% we try with the freedom degree rho=1/R0=1/ri :
    disp('Press enter to try now with no constraint on v0 and vi');
    rho = sdpvar(1,1,'full');    
    LMICA_0 = W0 - rho * v0*v0 * eye(n);
    LMICA_i = rho * vi*vi * eye(n) - Wi;
    LMI =  [Mat0<0, W0>0, Mati<0, Wi>0, LMICA < 0, LMICA_0 < 0, LMICA_i<0];
    solvesdp(LMI, [], opt);
    
    if max([max(eig(double(Mat0))), max(eig(double(Mati))), max(eig(double(LMICA))), max(eig(double(LMICA_0))), max(eig(double(LMICA_i)))])>0 
        % In that case the LMI couldn't be solved
        disp('Problem with the LMI (even with a freedom degree v');
        %%%%%% we stop.
        return
    else
        disp('R0 = ri = ')
        disp( 1/double(rho))        
    end
        
        
   
    if 0    
    % In that case the LMI couldn't be solved
    disp('Problem with the LMI');

    %%%%%% we try with the freedom degree v=R0=ri :
    disp('Press enter to try now with no constraint on v0 and vi');
    pause;
    LMI =  [Mat0<0, W0>0, Mati<0, Wi>0]; % We remove the constraint on the level curve of P0 and Pi
    solvesdp(LMI, [], opt);
    if max([max(eig(double(Mat0))), max(eig(double(Mati))) ])>0 
        disp('Problem with the LMI even when consideving no coveving assumption')
        disp('Problem with the computation of the local controller or of the non-local controller')
        return
    else
        %keyboard
        R0 = sdpvar(1,1,'full');
        ri = sdpvar(1,1,'full');
        LMI = [R0*double(Wi)-ri*double(W0)<0, R0>0, ri>0];
        solvesdp(LMI, [], opt);
        R0 = double(R0);
        ri = double(ri);
        v0sqr = sdpvar(1,1,'full');
        visqr = sdpvar(1,1,'full');
        LMICAsqr_0 = double(W0)-R0*v0sqr*eye(n);
        LMICAsqr_i = ri*visqr*eye(n)-double(Wi);
        
        LMI =  [LMICAsqr_0 <0, LMICAsqr_i  <0, v0sqr>0, visqr>0];
        solvesdp(LMI, [], opt);
        disp('end of computation of P0 and Pi found with some particular v0, vi, R0 and ri');
        v0 = sqrt(double(v0sqr))
        vi = sqrt(double(visqr))
    end
end
else
    disp('P0 and Pi satisfying the covering assumption found !!!');
end
pause;

% We store the solution in terms of P0, Pi, K0, Ki, J0, Ji
P0 = inv(double(W0));
K0 = double(H0) * P0;
J0 = double(J0);
Pi = inv(double(Wi));
Ki = double(Hi) * Pi;
Ji = double(Ji);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Uniting Controller %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Km = sdpvar(m,n,'full');
Jm = sdpvar(m,p,'full');

Q0 = [[N0'*M0 + M0'*N0 (-M0'-N0')]
    [(-M0-N0) 2*eye(p)]];
Qi = [[Ni'*Mi + Mi'*Ni (-Mi'-Ni')]
    [(-Mi-Ni) 2*eye(p)]];

MatUnit0 = [[P0*A+P0*B*Km+A'*P0+Km'*B'*P0 P0*B*Jm+P0*G]
    [(P0*B*Jm+P0*G)' zeros(p,p)]] - Q0- Qi;
MatUniti = [[Pi*A+Pi*B*Km+A'*Pi+Km'*B'*Pi Pi*B*Jm+Pi*G]
    [(Pi*B*Jm+Pi*G)' zeros(p,p)]] - Q0- Qi;
LMIUnit =[MatUnit0 < 0, MatUniti < 0];
solvesdp(LMIUnit, [], opt);
if max(eig(double(MatUnit0)))>0 disp('Problem with the Uniting LMI') 
    return
end
disp('Uniting LMI satisfied')