%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the M-files of a simulation in the paper entitled
% Global asymmptotic stabilizing feedback law for a system
% satifying two different sector conditions
% written by V. Andrieu, C Prieur, S. Tarbouriech, D. Arzelier
%
% Associated to Example 3
%
% Last modification 13/07/2010 by V. Andrieu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all

% settings of the solver
opt = sdpsettings('solver','sedumi','verbose',0);
% option_computation == 1, we first modify v0 and vi 
% option_computation == 2, we first modify R0 and ri 
option_computation = 2;


% Matrices defining the system
A = [0 1; 0 0];
B = [0 1]';
G = eye(2);
L = eye(2);

% Size of the matrices
[n,m] = size(B);
q = size(L,2);
p = size(G,1);

% Sector conditions defining the nonlinearities
M0 = eye(2);
Mi = [2 0; 0 0.5];

v0 = 10;
vi = 0.5;



% Local change of matvices (to get a LMI)
A0 = A + G * M0 * L;
S0 = -M0;
W0 = sdpvar(n,n,'symmetric');
H0 = sdpvar(m,n,'full');
J0 = sdpvar(m,p,'full');

% Matrix that has to be negative for the local design
Mat0 = [[A0*W0 + B*H0  + (A0*W0)'+ (B*H0)'  B*J0 + G + W0 * L' * S0']
    [(B*J0 + G + W0 * L' * S0')' -2*eye(p)]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Set-Global Controller %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% change of matrices (to get a LMI)
Ai = A + G * Mi * L;
Si = - Mi;
Wi = sdpvar(n,n,'symmetric');
Hi = sdpvar(m,n,'full');
Ji = sdpvar(m,p,'full');

% Matrix that has to be negative for the Set-Global design
Mati = [[Ai*Wi + B*Hi  + (Ai*Wi)'+ (B*Hi)'  B*Ji + G + Wi * L' * Si']
    [(B*Ji + G + Wi * L' * Si')' -2*eye(p)]];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Test Controlability
LMI =  [Mat0<0, W0>0, Mati<0, Wi>0];
solvesdp(LMI, [], opt);


% we check if the LMI was solved
if max([max(eig(double(Mat0))), max(eig(double(Mati)))])>0 
    % In that case the LMI couldn't be solved
    disp('Problem with the controller local or non local');
    return
else
    % In this case there exists a local and non local controller and we can
    % pursue the analysis
    disp('local and non local : OK');
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% We introduce the covering assumption analysis
rho = sdpvar(1,1,'full');
LMICA_0 = W0 - rho * v0*v0 * eye(n);
LMICA_i = rho * vi*vi * eye(n) - Wi;
LMICA = Wi-W0;
LMI =  [Mat0<0, W0>0, Mati<0, Wi>0, LMICA < 0, LMICA_0 < 0, LMICA_i<0];
solvesdp(LMI, [], opt);

% we check if the LMI was solved
if max([max(eig(double(Mat0))), max(eig(double(Mati))), max(eig(double(LMICA))), max(eig(double(LMICA_0))), max(eig(double(LMICA_i)))])>0 
    % In that case the LMI couldn't be solved
    disp('Problem with the covering assumption');
    %%%%%% we stop.
    return
end
disp('Coevering Assumption OK');

% We store the solution in terms of P0, Pi, K0, Ki, J0, Ji
P0 = inv(double(W0));
Pi = inv(double(Wi));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Uniting Controller %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Jm = sdpvar(m,p,'full');
Km = sdpvar(m,n,'full');

m1 = sdpvar(1,1);
m2 = sdpvar(1,1);

m3 = sdpvar(1,1);
m4 = sdpvar(1,1);

Q0 = [[M0-M0 (-M0')]
    [(-M0) 2*eye(p)]];
Qi = [[M0-M0 (-Mi')]
    [(-Mi) 2*eye(p)]];

MatUnit0 = [[P0*(A+B*Km)+(A+B*Km)'*P0 P0*(B*Jm+G)]
    [(P0*B*Jm+P0*G)' zeros(p,p)]] - m1*Q0- m2*Qi;
MatUniti = [[Pi*A+Pi*B*Km+A'*Pi+Km'*B'*Pi Pi*B*Jm+Pi*G]
    [(Pi*B*Jm+Pi*G)' zeros(p,p)]] - m3*Q0- m4*Qi;
LMIUnit =[MatUnit0 < 0, MatUniti < 0];
solvesdp(LMIUnit, [], opt);
if max(eig(double(MatUnit0)))>0 disp('Problem with the Uniting LMI in two step') 
else
    disp('Uniting LMI in two step satisfied')
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Now we check for the solvability
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% of the uniting LMI in ond step

G1 = sdpvar(n,n,'full');
Hm = sdpvar(m,n,'full');

mu1 = 1;%double(m1);
mu2 = 1;%double(m2);
mu3 = 1;%double(m3);
mu4 = 1;%double(m4);


MUnit0 = [[(A*G1+B*Hm) + (A*G1+B*Hm)' (G1-W0+A*G1+B*Hm)' (mu1*M0*G1+mu2*Mi*G1 + Jm'*B' + G' + A*G1+B*Hm)']
    [(G1-W0+A*G1+B*Hm) (-2*W0) (B*Jm +G -W0)]
    [(mu1*M0*G1+mu2*Mi*G1 + Jm'*B' + G' + A*G1+B*Hm) (B*Jm +G -W0)' (-2*(mu1 + mu2)*eye(n) - B*Jm - G - Jm'*B' -G')]]; 


MUniti = [[(A*G1+B*Hm) + (A*G1+B*Hm)' (G1-Wi+A*G1+B*Hm)' (mu3*M0*G1+mu4*Mi*G1 + Jm'*B' + G' + A*G1+B*Hm)']
    [(G1-Wi+A*G1+B*Hm) (-2*Wi) (B*Jm +G -Wi)]
    [(mu3*M0*G1+mu4*Mi*G1 + Jm'*B' + G' + A*G1+B*Hm) (B*Jm +G -Wi)' (-2*(mu3+mu4)*eye(n) - B*Jm - G - Jm'*B' -G')]]; 
%
LMI =  [W0>0,  Wi>0,  MUnit0 < 0, Mat0<0 ,Mati<0, LMICA < 0, LMICA_0 < 0, LMICA_i<0, MUniti < 0];
solvesdp(LMI, [], opt);

if max(eig(double(MUnit0)))>0 disp('Problem with the Uniting LMI in one step') 
    return
end
disp('Uniting LMI satisfied in one step !!!')